==> botanypots-fabric-1.21.1-21.1.8-sources.jar: net/darkhax/botanypots/common/impl/data/itemdrops/LootTableDrops.java <==
package net.darkhax.botanypots.common.impl.data.itemdrops;

import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.darkhax.bookshelf.common.api.function.CachedSupplier;
import net.darkhax.bookshelf.common.api.loot.LootPoolEntryDescriptions;
import net.darkhax.botanypots.common.api.context.BotanyPotContext;
import net.darkhax.botanypots.common.api.data.itemdrops.ItemDropProvider;
import net.darkhax.botanypots.common.api.data.itemdrops.ItemDropProviderType;
import net.darkhax.botanypots.common.impl.BotanyPotsMod;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2960;
import net.minecraft.class_52;
import net.minecraft.class_5321;
import net.minecraft.class_7924;
import net.minecraft.class_8567;
import net.minecraft.class_9129;
import net.minecraft.class_9139;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class LootTableDrops implements ItemDropProvider {

    public static final Supplier<ItemDropProviderType<?>> TYPE = ItemDropProviderType.getLazy(BotanyPotsMod.id("loot_table"));
    public static final MapCodec<LootTableDrops> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
            class_2960.field_25139.fieldOf("table_id").forGetter(LootTableDrops::getTableId)
    ).apply(instance, LootTableDrops::new));
    public static final class_9139<class_9129, LootTableDrops> STREAM = class_9139.method_56437(
            (buf, val) -> {
                buf.method_10812(val.tableId);
                class_1799.field_49269.encode(buf, val.displayItems.get());
            },
            buf -> {
                final class_2960 tableId = buf.method_10810();
                final List<class_1799> items = class_1799.field_49269.decode(buf);
                return new LootTableDrops(tableId, items);
            }
    );

    private final class_2960 tableId;
    private final CachedSupplier<class_52> cachedTable;
    private final CachedSupplier<List<class_1799>> displayItems;

    // Server
    public LootTableDrops(class_2960 tableId) {
        this.tableId = tableId;
        this.cachedTable = CachedSupplier.cache(() -> Objects.requireNonNull(BotanyPotsMod.REGISTRY_ACCESS.get()).method_30530(class_7924.field_50079).method_29107(class_5321.method_29179(class_7924.field_50079, tableId)));
        this.displayItems = CachedSupplier.cache(() -> LootPoolEntryDescriptions.getUniqueItems(Objects.requireNonNull(BotanyPotsMod.REGISTRY_ACCESS.get()), this.cachedTable.get()));
    }

    // client
    public LootTableDrops(class_2960 tableId, List<class_1799> displayItems) {
        this.tableId = tableId;
        this.cachedTable = CachedSupplier.singleton(null);
        this.displayItems = CachedSupplier.singleton(displayItems);
    }

    public class_2960 getTableId() {
        return this.tableId;
    }

    protected class_8567 getLootParams(BotanyPotContext context) {
        return context.createLootParams(null);
    }

    @Override
    public void apply(BotanyPotContext context, class_1937 level, Consumer<class_1799> drops) {
        this.cachedTable.ifPresent(table -> table.method_51882(this.getLootParams(context), drops));
    }

    @Override
    public ItemDropProviderType<?> getType() {
        return TYPE.get();
    }

    @Override
    public final List<class_1799> getDisplayItems() {
        return this.displayItems.get();
    }
}

==> botanypots-fabric-1.21.1-21.1.25-sources.jar: net/darkhax/botanypots/common/impl/data/itemdrops/LootTableDrops.java <==
package net.darkhax.botanypots.common.impl.data.itemdrops;

import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import net.darkhax.bookshelf.common.api.function.CachedSupplier;
import net.darkhax.bookshelf.common.api.loot.LootPoolEntryDescriptions;
import net.darkhax.botanypots.common.api.context.BotanyPotContext;
import net.darkhax.botanypots.common.api.data.itemdrops.ItemDropProvider;
import net.darkhax.botanypots.common.api.data.itemdrops.ItemDropProviderType;
import net.darkhax.botanypots.common.impl.BotanyPotsMod;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2960;
import net.minecraft.class_52;
import net.minecraft.class_5321;
import net.minecraft.class_7924;
import net.minecraft.class_8567;
import net.minecraft.class_9129;
import net.minecraft.class_9139;
import java.util.List;
import java.util.Objects;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class LootTableDrops implements ItemDropProvider {

    public static final Supplier<ItemDropProviderType<?>> TYPE = ItemDropProviderType.getLazy(BotanyPotsMod.id("loot_table"));
    public static final MapCodec<LootTableDrops> CODEC = RecordCodecBuilder.mapCodec(instance -> instance.group(
            class_2960.field_25139.fieldOf("table_id").forGetter(LootTableDrops::getTableId)
    ).apply(instance, LootTableDrops::new));
    public static final class_9139<class_9129, LootTableDrops> STREAM = class_9139.method_56437(
            (buf, val) -> {
                buf.method_10812(val.tableId);
                class_1799.field_49269.encode(buf, val.displayItems.get());
            },
            buf -> {
                final class_2960 tableId = buf.method_10810();
                final List<class_1799> items = class_1799.field_49269.decode(buf);
                return new LootTableDrops(tableId, items);
            }
    );

    private final class_2960 tableId;
    private final CachedSupplier<class_52> cachedTable;
    private final CachedSupplier<List<class_1799>> displayItems;

    // Server
    public LootTableDrops(class_2960 tableId) {
        this.tableId = tableId;
        this.cachedTable = CachedSupplier.cache(() -> Objects.requireNonNull(BotanyPotsMod.REGISTRY_ACCESS.get()).method_30530(class_7924.field_50079).method_29107(class_5321.method_29179(class_7924.field_50079, tableId)));
        this.displayItems = CachedSupplier.cache(() -> LootPoolEntryDescriptions.getUniqueItems(Objects.requireNonNull(BotanyPotsMod.REGISTRY_ACCESS.get()), this.cachedTable.get()));
    }

    // client
    public LootTableDrops(class_2960 tableId, List<class_1799> displayItems) {
        this.tableId = tableId;
        this.cachedTable = CachedSupplier.singleton(null);
        this.displayItems = CachedSupplier.singleton(displayItems);
    }

    public class_2960 getTableId() {
        return this.tableId;
    }

    protected class_8567 getLootParams(BotanyPotContext context) {
        return context.createLootParams(null);
    }

    @Override
    public void apply(BotanyPotContext context, class_1937 level, Consumer<class_1799> drops) {
        if (hasTableDrops(level)) {
            this.cachedTable.get().method_51882(this.getLootParams(context), drops);
        }
        else {
            fallbackDrops(context, level, drops);
        }
    }

    public void fallbackDrops(BotanyPotContext context, class_1937 level, Consumer<class_1799> drops) {
    }

    public boolean hasTableDrops(class_1937 level) {
        final class_52 table = this.cachedTable.get();
        return table != null && table != class_52.field_948;
    }

    @Override
    public ItemDropProviderType<?> getType() {
        return TYPE.get();
    }

    @Override
    public final List<class_1799> getDisplayItems() {
        return this.displayItems.get();
    }
}
